import { IProjectBuilder, IProjectBuilderOptions } from '../types';

import { createProjectBuilder } from '../generator/ProjectBuilder';

import esmodule from '../plugins/common/esmodule';
import styleImport from '../plugins/common/styleImport';
import containerClass from '../plugins/component/myreact/containerClass';
import containerInitState from '../plugins/component/myreact/containerInitState';
import containerInjectContext from '../plugins/component/myreact/containerInjectContext';
import containerInjectUtils from '../plugins/component/myreact/containerInjectUtils';
import containerInjectDataSourceEngine from '../plugins/component/myreact/containerInjectDataSourceEngine';
import containerInjectConstants from '../plugins/component/myreact/containerInjectConstants';
import containerInjectI18n from '../plugins/component/myreact/containerInjectI18n';
import containerLifeCycle from '../plugins/component/myreact/containerLifeCycle';
import containerMethod from '../plugins/component/myreact/containerMethod';
import jsx from '../plugins/component/myreact/jsx';
import reactCommonDeps from '../plugins/component/myreact/reactCommonDeps';
import css from '../plugins/component/style/css';
import constants from '../plugins/project/constants';
import i18n from '../plugins/project/i18n';
import utils from '../plugins/project/utils';

import icejszwh from '../plugins/project/framework/icejszwh';

import { prettier } from '../postprocessor';

export type IceJsProjectBuilderOptions = IProjectBuilderOptions;

export default function createIceJsZwhProjectBuilder(
  options?: IceJsProjectBuilderOptions
): IProjectBuilder {
  return createProjectBuilder({
    inStrictMode: options?.inStrictMode,
    extraContextData: { ...options },
    template: icejszwh.template, // react 项目模板，可以不需要修改
    plugins: {
      components: [
        reactCommonDeps(),
        esmodule({
          fileType: 'tsx',
        }),
        styleImport(),
        containerClass(),
        // containerInjectContext(),
        // containerInjectUtils(),
        containerInjectDataSourceEngine(),
        // containerInjectI18n(),
        containerInjectConstants(),
        containerInitState(),
        containerLifeCycle(),
        containerMethod(),
        jsx({
          nodeTypeMapping: {
            Div: 'div',
            Component: 'div',
            Page: 'div',
            Block: 'div',
          },
        }),
        css(),
      ],
      pages: [
        reactCommonDeps(),
        esmodule({
          fileType: 'tsx',
        }),
        styleImport(),
        containerClass(),
        // containerInjectContext(),
        // containerInjectUtils(),
        // containerInjectDataSourceEngine(),
        // containerInjectI18n(),
        // containerInjectConstants(),
        // containerInitState(),
        // containerLifeCycle(),
        // containerMethod(),
        jsx({
          nodeTypeMapping: {
            Div: 'div',
            Component: 'div',
            Page: 'div',
            Block: 'div',
            // Box: 'div',
          },
        }),
        css(),
      ],
      router: [esmodule(), icejszwh.plugins.router()],
      entry: [icejszwh.plugins.entry()],
      constants: [constants()],
      utils: [esmodule(), utils('react')],
      i18n: [i18n()],
      globalStyle: [icejszwh.plugins.globalStyle()],
      htmlEntry: [icejszwh.plugins.entryHtml()],
      packageJSON: [icejszwh.plugins.packageJSON()],
    },
    postProcessors: [prettier()],
    customizeBuilderOptions: options?.customizeBuilderOptions,
  });
}

export const plugins = {
  containerClass,
  // containerInjectContext,
  // containerInjectUtils,
  // containerInjectDataSourceEngine,
  // containerInjectI18n,
  // containerInjectConstants,
  // containerInitState,
  // containerLifeCycle,
  // containerMethod,
  jsx,
  commonDeps: reactCommonDeps,

  /** @deprecated Please use commonDeps */
  reactCommonDeps,
};
